@@ -1,5 +1,5 @@
 /* install - copy files and set attributes
-   Copyright (C) 1989-2015 Free Software Foundation, Inc.
+   Copyright (C) 1989-2016 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,7 +38,6 @@
 #include "modechange.h"
 #include "prog-fprintf.h"
 #include "quote.h"
-#include "quotearg.h"
 #include "savewd.h"
 #include "stat-time.h"
 #include "utimens.h"
@@ -378,7 +377,7 @@ setdefaultfilecon (char const *file)
   if (lsetfilecon (file, scontext) < 0 && errno != ENOTSUP)
     error (0, errno,
            _("warning: %s: failed to change context to %s"),
-           quotearg_colon (file), scontext);
+           quotef_n (0, file), quote_n (1, scontext));
 
   freecon (scontext);
   return;
@@ -406,9 +405,10 @@ target_directory_operand (char const *file)
   int err = (stat (file, &st) == 0 ? 0 : errno);
   bool is_a_dir = !err && S_ISDIR (st.st_mode);
   if (err && err != ENOENT)
-    error (EXIT_FAILURE, err, _("failed to access %s"), quote (file));
+    error (EXIT_FAILURE, err, _("failed to access %s"), quoteaf (file));
   if (is_a_dir < looks_like_a_dir)
-    error (EXIT_FAILURE, err, _("target %s is not a directory"), quote (file));
+    error (EXIT_FAILURE, err, _("target %s is not a directory"),
+           quoteaf (file));
   return is_a_dir;
 }
 
@@ -418,7 +418,7 @@ announce_mkdir (char const *dir, void *options)
 {
   struct cp_options const *x = options;
   if (x->verbose)
-    prog_fprintf (stdout, _("creating directory %s"), quote (dir));
+    prog_fprintf (stdout, _("creating directory %s"), quoteaf (dir));
 }
 
 /* Make ancestor directory DIR, whose last file name component is
@@ -486,9 +486,9 @@ change_attributes (char const *name)
 
   if (! (owner_id == (uid_t) -1 && group_id == (gid_t) -1)
       && lchown (name, owner_id, group_id) != 0)
-    error (0, errno, _("cannot change ownership of %s"), quote (name));
+    error (0, errno, _("cannot change ownership of %s"), quoteaf (name));
   else if (chmod (name, mode) != 0)
-    error (0, errno, _("cannot change permissions of %s"), quote (name));
+    error (0, errno, _("cannot change permissions of %s"), quoteaf (name));
   else
     ok = true;
 
@@ -510,7 +510,7 @@ change_timestamps (struct stat const *src_sb, char const *dest)
 
   if (utimens (dest, timespec))
     {
-      error (0, errno, _("cannot set time stamps for %s"), quote (dest));
+      error (0, errno, _("cannot set time stamps for %s"), quoteaf (dest));
       return false;
     }
   return true;
@@ -546,7 +546,7 @@ strip (char const *name)
       }
 #endif /* __CYGWIN__ */
       execlp (strip_program, strip_program, name, NULL);
-      error (EXIT_FAILURE, errno, _("cannot run %s"), strip_program);
+      error (EXIT_FAILURE, errno, _("cannot run %s"), quoteaf (strip_program));
       break;
     default:			/* Parent. */
       if (waitpid (pid, &status, 0) < 0)
@@ -576,7 +576,8 @@ get_ids (void)
           unsigned long int tmp;
           if (xstrtoul (owner_name, NULL, 0, &tmp, NULL) != LONGINT_OK
               || UID_T_MAX < tmp)
-            error (EXIT_FAILURE, 0, _("invalid user %s"), quote (owner_name));
+            error (EXIT_FAILURE, 0, _("invalid user %s"),
+                   quote (owner_name));
           owner_id = tmp;
         }
       else
@@ -594,7 +595,8 @@ get_ids (void)
           unsigned long int tmp;
           if (xstrtoul (group_name, NULL, 0, &tmp, NULL) != LONGINT_OK
               || GID_T_MAX < tmp)
-            error (EXIT_FAILURE, 0, _("invalid group %s"), quote (group_name));
+            error (EXIT_FAILURE, 0, _("invalid group %s"),
+                   quote (group_name));
           group_id = tmp;
         }
       else
@@ -644,6 +646,7 @@ In the 4th form, create all components of the given DIRECTORY(ies).\n\
 "), stdout);
       fputs (_("\
   -D                  create all leading components of DEST except the last,\n\
+                        or all components of --target-directory,\n\
                         then copy SOURCE to DEST\n\
   -g, --group=GROUP   set group ownership, instead of process' current group\n\
   -m, --mode=MODE     set permission mode (as in chmod), instead of rwxr-xr-x\n\
@@ -698,7 +701,7 @@ install_file_in_file (const char *from, const char *to,
   struct stat from_sb;
   if (x->preserve_timestamps && stat (from, &from_sb) != 0)
     {
-      error (0, errno, _("cannot stat %s"), quote (from));
+      error (0, errno, _("cannot stat %s"), quoteaf (from));
       return false;
     }
   if (! copy_file (from, to, x))
@@ -707,7 +710,7 @@ install_file_in_file (const char *from, const char *to,
     if (! strip (to))
       {
         if (unlink (to) != 0)  /* Cleanup.  */
-          error (EXIT_FAILURE, errno, _("cannot unlink %s"), to);
+          error (EXIT_FAILURE, errno, _("cannot unlink %s"), quoteaf (to));
         return false;
       }
   if (x->preserve_timestamps && (strip_files || ! S_ISREG (from_sb.st_mode))
@@ -716,14 +719,17 @@ install_file_in_file (const char *from, const char *to,
   return change_attributes (to);
 }
 
-/* Copy file FROM onto file TO, creating any missing parent directories of TO.
+/* Create any missing parent directories of TO,
+   while maintaining the current Working Directory.
    Return true if successful.  */
 
 static bool
-mkancesdirs_safe_wd (char const *from, char *to, struct cp_options *x)
+mkancesdirs_safe_wd (char const *from, char *to, struct cp_options *x,
+                     bool save_always)
 {
   bool save_working_directory =
-    ! (IS_ABSOLUTE_FILE_NAME (from) && IS_ABSOLUTE_FILE_NAME (to));
+    save_always
+    || ! (IS_ABSOLUTE_FILE_NAME (from) && IS_ABSOLUTE_FILE_NAME (to));
   int status = EXIT_SUCCESS;
 
   struct savewd wd;
@@ -733,7 +739,7 @@ mkancesdirs_safe_wd (char const *from, char *to, struct cp_options *x)
 
   if (mkancesdirs (to, &wd, make_ancestor, x) == -1)
     {
-      error (0, errno, _("cannot create directory %s"), to);
+      error (0, errno, _("cannot create directory %s"), quoteaf (to));
       status = EXIT_FAILURE;
     }
 
@@ -746,7 +752,8 @@ mkancesdirs_safe_wd (char const *from, char *to, struct cp_options *x)
         return false;
       if (restore_result < 0 && status == EXIT_SUCCESS)
         {
-          error (0, restore_errno, _("cannot create directory %s"), to);
+          error (0, restore_errno, _("cannot create directory %s"),
+                 quoteaf (to));
           return false;
         }
     }
@@ -760,7 +767,7 @@ static bool
 install_file_in_file_parents (char const *from, char *to,
                               const struct cp_options *x)
 {
-  return (mkancesdirs_safe_wd (from, to, (struct cp_options *)x)
+  return (mkancesdirs_safe_wd (from, to, (struct cp_options *)x, false)
           && install_file_in_file (from, to, x));
 }
 
@@ -777,7 +784,7 @@ install_file_in_dir (const char *from, const char *to_dir,
   bool ret = true;
 
   if (mkdir_and_install)
-    ret = mkancesdirs_safe_wd (from, to, (struct cp_options *)x);
+    ret = mkancesdirs_safe_wd (from, to, (struct cp_options *)x, true);
 
   ret = ret && install_file_in_file (from, to, x);
   free (to);
@@ -939,10 +946,10 @@ main (int argc, char **argv)
       bool stat_success = stat (target_directory, &st) == 0 ? true : false;
       if (! mkdir_and_install && ! stat_success)
         error (EXIT_FAILURE, errno, _("failed to access %s"),
-               quote (target_directory));
+               quoteaf (target_directory));
       if (stat_success && ! S_ISDIR (st.st_mode))
         error (EXIT_FAILURE, 0, _("target %s is not a directory"),
-               quote (target_directory));
+               quoteaf (target_directory));
     }
 
   if (backup_suffix_string)
@@ -971,7 +978,7 @@ main (int argc, char **argv)
         error (0, 0, _("missing file operand"));
       else
         error (0, 0, _("missing destination file operand after %s"),
-               quote (file[0]));
+               quoteaf (file[0]));
       usage (EXIT_FAILURE);
     }
 
@@ -983,7 +990,7 @@ main (int argc, char **argv)
                  "and --no-target-directory (-T)"));
       if (2 < n_files)
         {
-          error (0, 0, _("extra operand %s"), quote (file[2]));
+          error (0, 0, _("extra operand %s"), quoteaf (file[2]));
           usage (EXIT_FAILURE);
         }
     }
@@ -993,7 +1000,7 @@ main (int argc, char **argv)
         target_directory = file[--n_files];
       else if (2 < n_files)
         error (EXIT_FAILURE, 0, _("target %s is not a directory"),
-               quote (file[n_files - 1]));
+               quoteaf (file[n_files - 1]));
     }
 
   if (specified_mode)
@@ -1051,7 +1058,7 @@ main (int argc, char **argv)
           dest_info_init (&x);
           for (i = 0; i < n_files; i++)
             if (! install_file_in_dir (file[i], target_directory, &x,
-                                       mkdir_and_install))
+                                       i == 0 && mkdir_and_install))
               exit_status = EXIT_FAILURE;
         }
     }
